<?php
/***************************************************************************
                                smartfeed.php
                             -------------------
    begin                : Thurs Nov 24 2005
    copyright            : (C) 2000 The phpBB Group
    email                : mhamill@computer.org

    $Id: $

***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 ***************************************************************************/

// Written by Mark D. Hamill, mhamill@computer.org
// This software is designed to work with phpBB Version 2.0.18

// This is the program that creates the RSS or Atom compliant newsfeed that will also include protected
// forums. An enhanced and slightly modified version of the the FeedCreator class, version 1.7.2 was used 
// to generate the various feeds. The modified version can be found at http://blog.mypapit.net/2005/11/using-feedcreator-to-generate-atom-10-feeds.html.
// This version is needed because it supports Atom 1.0 and allow feeds to be created on the fly, 
// instead of stored to files.
//
// A companion program should be run first: smartfeed_url.php. This creates a URL that is
// used to invoke the feed. Use the URL created by this program in your newsreader. Do NOT try to run 
// this program without arguments!

define('IN_PHPBB', true);
$phpbb_root_path = './';

include($phpbb_root_path . 'extension.inc');
include($phpbb_root_path . 'common.'.$phpEx);
include($phpbb_root_path . 'includes/bbcode.'.$phpEx);

// Override the Feed Creator Generator information to include info on this modification
define("FEEDCREATOR_VERSION", "FeedCreator 1.7.2-ppt (info@mypapit.net) and SmartFeed phpBB Modification (mhamill@computer.org)");
// Override the Feed Creator timezone to reflect the board timezone
define("TIME_ZONE",str_replace('.',':',sprintf("%+'06.2f",$board_config['board_timezone'])));

include($phpbb_root_path . 'includes/feedcreator.class.' . $phpEx); 

// Code from rss.php Mod by Sascha Carlin. Create an absolute URL to the site. 
$script_name = preg_replace('/^\/?(.*?)\/?$/', '\1', trim($board_config['script_path']));
$server_name = trim($board_config['server_name']);
$server_protocol = ( $board_config['cookie_secure'] ) ? 'https://' : 'http://';
$server_port = ( $board_config['server_port'] <> 80 ) ? ':' . trim($board_config['server_port']) . '/' : '/';
$siteURL = $server_protocol . $server_name . $server_port . $script_name;
// End copied code

$userdata = session_pagestart($user_ip, PAGE_INDEX);
init_userprefs($userdata);

include($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_smartfeed.' . $phpEx);

// Set up some needed variables
$error_lang = '';  // Used to capture any errors so the user can see they goofed. If so they get HTML instead of XML
$limit_str = '';
$error = false;
$reset = false;

// Format Timezone for display
$l_timezone = explode('.', $board_config['board_timezone']);
$l_timezone = (count($l_timezone) > 1 && $l_timezone[count($l_timezone)-1] != 0) ? $lang[sprintf('%.1f', $board_config['board_timezone'])] : $lang[number_format($board_config['board_timezone'])];

// Check for the u argument. u corresponds to the user_id wanted
$user_id = $_GET['u'];
if ($user_id == '')
{
	$error_lang = $lang['smartfeed_no_u_param'];
	$error = true;
}
	
// Check for the p argument. p corresponds to the encrypted password in the phpBB database, which is a hash value
$encrypted_pswd = $_GET['p'];
if ($encrypted_pswd == '')
{
	$error_lang .= $lang['smartfeed_no_p_param'];
	$error = true;
}

// Validate user id and password
if (!$error) 
{
	// Make sure user_id exists in database
	$sql = "SELECT count(*) as count
		FROM " . USERS_TABLE . "
		WHERE user_id = '" . $user_id . "'";
		
	if ( !($result = $db->sql_query($sql)) )
	{
		$error_lang = $lang['smartfeed_user_table_count_error'] ;
		$error = true;
	}
	else
	{
		$row = $db->sql_fetchrow($result);
		
		if ($row['count'] == 0)
		{
			$error_lang = sprintf($lang['smartfeed_user_id_does_not_exist'], $user_id);
			$error = true;
		}
		else
		{
			// Validate encrypted password
			$sql = "SELECT user_password
				FROM " . USERS_TABLE . "
				WHERE user_id = '" . $user_id . "'";
			
			if ( !($result = $db->sql_query($sql)) )
			{
				$error_lang = $lang['smartfeed_user_table_password_error'];
				$error = true;
			}
			else
			{
				$row = $db->sql_fetchrow($result);
				if ($encrypted_pswd <> $row['user_password'])
				{
					$error_lang = sprintf($lang['smartfeed_bad_password_error'],  $encrypted_pswd, $user_id);
					$error = true;
				}
				else
				{
					// Check to see if last visit time should be reset
					$lastvisit = $_GET['lastvisit'];
					if (($lastvisit <> '') && ($lastvisit <> '1'))
					{
						$error_lang = $lang['smartfeed_lastvisit_param'];
						$error = true;
					}
					else
					{
						$reset = true;
					}	
				}			
			}
		}
	}
}

if (!$error)
{

	// Get all forum_ids that this user is allowed to access
	
	$sql = 'SELECT f.forum_id, f.forum_name, c.cat_order, f.forum_order
		FROM ' . FORUMS_TABLE . ' f, ' . CATEGORIES_TABLE . ' c
		WHERE f.cat_id = c.cat_id AND auth_read IN (' . AUTH_ALL. ',' . AUTH_REG .') 
		ORDER BY c.cat_order, f.forum_order';

	if ( !($result = $db->sql_query($sql)))
	{
		$error_lang = $lang['smartfeed_forum_access_reg'];
		$error = true;
	}

	if (!$error)
	{
		$allowed_forums = array();
		while ($row = $db->sql_fetchrow ($result)) 
		{ 
			$allowed_forums[] = $row['forum_id'];
		}
		$db->sql_freeresult ($result);
	
		$sql = 'SELECT DISTINCT a.forum_id, f.forum_name, c.cat_order, f.forum_order
			FROM ' . AUTH_ACCESS_TABLE . ' a, ' . USER_GROUP_TABLE . ' ug, ' . FORUMS_TABLE . ' f, ' . CATEGORIES_TABLE . ' c
			WHERE ug.user_id = ' . $user_id
			. ' AND ug.user_pending = 0 
			AND a.group_id = ug.group_id AND 
			a.forum_id = f.forum_id AND f.cat_id = c.cat_id';
	
		if ( !($result = $db->sql_query($sql)))
		{
			$error_lang = $lang['smartfeed_forum_access_priv'];
			$error = true;
		}
		
		if (!$error)
		{
	
			while ($row = $db->sql_fetchrow ($result)) 
			{ 
				$allowed_forums[] = $row['forum_id'];
			}
			$db->sql_freeresult ($result);
		
			// Sort forums by forum_id and ensure there are no duplicates
		
			asort($allowed_forums);
			$allowed_forums = array_unique($allowed_forums);

		}
	}
}

if (!$error)
{

	// Now parse the URL field and get the forums the user wants to view
	
	$requested_forums = array();
	$params = explode('&', $_SERVER['QUERY_STRING']);
	foreach ($params as $item)
	{
		if (substr($item,0,5) == 'forum')
			$requested_forums[] = substr($item,6);
	}
	asort($requested_forums);
	$requested_forums = array_unique($requested_forums);

	// The forums that will be fetched is the intersection of the requested and allowed forums. This prevents hacking
	// the URL to get feeds a user is not supposed to get. If no forums are specified on the URL field
	// then all forums that this user is authorized to access is assumed.
	
	if (count($requested_forums) > 0)
	{
		$fetched_forums = array_intersect($allowed_forums, $requested_forums);
	}
	else
	{
		$fetched_forums = $allowed_forums;
	}
	
	// Place forum numbers into a string suitable for use in a SQL "IN" statement

	$fetched_forums_str = implode(',',$fetched_forums);
	if ($fetched_forums_str <> '')
	{
		$fetched_forums_str = ' AND f.forum_id in (' . $fetched_forums_str . ')';
	}

	// Get the limit parameter. It limits the size of the newsfeed to a point in time from the present, either a day/hour/minute interval, 
	// or the time since the user's last visit. It should always exist.
	$limit = rawurldecode($_GET['limit']);
	if ($limit == 'LF')  // LF = Last Fetch, i.e. use the user's last visit and show all selected items after this time in the feed
	{
		// Logic to retrieve last fetched date
		$sql = 'SELECT user_lastvisit
			FROM ' . USERS_TABLE . "
			WHERE user_id = '" . $user_id . "'";
	
		if ( !($result = $db->sql_query($sql)))
		{
			$error_lang = $lang['smartfeed_user_error'];
			$error = true;
		}
		
		if (!$error)
		{
			$row = $db->sql_fetchrow ($result);
			$limit_text = ' AND p.post_time > ' . $row['user_lastvisit'];
			$db->sql_freeresult ($result);
		}		
	}
	else if (($limit =='7 DAY') || ($limit =='1 DAY') || ($limit =='12 HOUR') || ($limit =='6 HOUR') || ($limit =='3 HOUR') || ($limit =='1 HOUR') || ($limit =='30 MINUTE') || ($limit =='15 MINUTE'))
	{
		$limit_text = ' AND p.post_time > UNIX_TIMESTAMP(DATE_SUB(NOW(), INTERVAL ' . $limit . '))';
	}
	else
	{
		// Bad parameter, trigger error
		$error_lang = $lang['smartfeed_limit_format_error'];
		$error = true;
	}
	
	// Validate the feed type next
	if (!$error)
	{
		$feed_type = rawurldecode($_GET['feed_type']);
		if (!(($feed_type == $lang['smartfeed_atom_10_value']) || ($feed_type == $lang['smartfeed_rss_20_value']) || ($feed_type == $lang['smartfeed_rss_10_value']) || ($feed_type == $lang['smartfeed_rss_091_value'])))
		{
			// Bad feed_type, trigger error
			$error_lang = $lang['smartfeed_feed_type_error'];
			$error = true;
		}
		else
		{
			$limit_str = ($feed_type == 'RSS0.91') ? ' LIMIT 15' : '';
		}
	}
	
	
	
	// Validate the sort by parameter
	if (!$error)
	{
		$order_by = rawurldecode($_GET['sort_by']);
		if ($order_by == 'standard')
		{
			$order_by_str = 'c.cat_order, f.forum_order, lp.post_time desc, p.post_time';
		}
		else if ($order_by == 'postdate')
		{
			$order_by_str = 'p.post_time';
		}
		else
		{
			// Bad sort_bye, trigger error
			$error_lang = $lang['smartfeed_sort_by_error'];
			$error = true;
		}
	}
	
	// Validate the new topics only parameter. If it does not show, it is not true. Any value other than 1 is false.
	if (!$error)
	{
		$order_by = rawurldecode($_GET['topicsonly']);
		if ($order_by == '')
		{
			$topics_only_str = '';
		}
		else if ($order_by == '1')
		{
			$topics_only_str = ' AND t.topic_time > UNIX_TIMESTAMP(DATE_SUB(NOW(), INTERVAL ' . $limit . '))';
		}
		else
		{
			// Bad sort_bye, trigger error
			$error_lang = $lang['smartfeed_topics_only_error'];
			$error = true;
		}
	}
			
	if (!$error)
	{

		// Create a list of messages for this user that presumably have not been seen after the date and time requested by the limit parameter.
		// Show only authorized forums.
		
		$sql = "SELECT c.cat_title, f.forum_name, t.topic_title, t.topic_id, p.post_time, from_unixtime(p.post_time, '%d %b %Y, %h:%i %p') AS 'Post Time', 
			u.username, pt.post_text, pt.post_subject, pt.post_id, t.topic_first_post_id, u.user_viewemail, u.user_email, u.user_sig, pt.bbcode_uid, u.user_sig_bbcode_uid
			FROM " . POSTS_TABLE . ' lp, ' . FORUMS_TABLE . ' f, ' . TOPICS_TABLE . ' t, ' . CATEGORIES_TABLE . ' c, ' . POSTS_TABLE . ' p, ' . POSTS_TEXT_TABLE . ' pt, ' . USERS_TABLE . ' u
			WHERE lp.post_id = t.topic_last_post_id AND f.forum_id = t.forum_id AND
			f.cat_id = c.cat_id AND t.topic_id = p.topic_id AND p.post_id = pt.post_id AND p.poster_id = u.user_id ' . $limit_text . $fetched_forums_str . $topics_only_str . '
			ORDER BY ' . $order_by_str . $limit_str; 
	
		if ( !($result = $db->sql_query($sql)))
		{
			$error_lang = $lang['smartfeed_retrieve_error'];
			$error = true;
		}
		else
		{
			$rss = new UniversalFeedCreator();
			$rss->useCached();
			$rss->title = $lang['smartfeed_feed_title'];
			if ($feed_type=='RSS0.91') // limit of 500 characters, not handled by class
			{
				$rss->description = substr($lang['smartfeed_feed_description'],0,500);
				$rss->link = substr( $siteURL . '?' . htmlspecialchars($_SERVER['QUERY_STRING']),0,500);
			}
			else
			{
				$rss->description = $lang['smartfeed_feed_description'];
				$rss->link = $siteURL . '?' . htmlspecialchars($_SERVER['QUERY_STRING']);
			}
			
			$rss->syndicationURL = $siteURL . 'smartfeed.php';
			
			$image = new FeedImage();
			$image->title = strip_tags($lang['smartfeed_image_title']); // Enhance to use default logo if not specified
			$image->url = $siteURL . $smartfeed_feed_image_path;
			$image->link = $siteURL;
			$image->description = strip_tags($board_config['site_desc']);
			$rss->image = $image;
			
			// Some RSS 2.0 tags
			$rss->language = $lang['smartfeed_rfc1766_lang'];
			$rss->pubDate = time();
			$rss->ttl = $lang['smartfeed_ttl'];
			
			if ($lang['smartfeed_copyright'] <> '')
			{
				$rss->copyright = $lang['smartfeed_copyright'];
			}
			if ($lang['smartfeed_editor'] <> '')
			{
				$rss->editor = $lang['smartfeed_editor'];
			}
			if ($lang['smartfeed_webmaster'] <> '')
			{
				$rss->smartfeed_webmaster = $lang['smartfeed_webmaster'];
			}
			
			while ($row = $db->sql_fetchrow ($result)) 
			{ 

				$item = new FeedItem();
				$item->title = ($row['post_subject'] == '') ? html_entity_decode($row['forum_name'] . ' :: ' . $row['topic_title']) : html_entity_decode($row['forum_name'] . ' :: ' . $row['post_subject']);
				if ($row['topic_first_post_id'] <> $row['post_id'])
				{
					$item->title .= ' ' . $lang['smartfeed_reply'];
				}
				$item->link = $siteURL . 'viewtopic.' . $phpEx . '?' . POST_POST_URL . '=' . $row['post_id'] . '#' . $row['post_id'];
				$item->date = $row['post_time'];
				$item->pubDate = $row['post_time'];
				$item->source = $siteURL;
				$item->category = $row['cat_title'];
				$item->guid = $item->link;
				$item->comments = $siteURL . 'posting.php?mode=reply&t=' . $row['topic_id'];
				if (($feed_type=='RSS0.91') || ($feed_type=='RSS2.0')) // RSS 0.91 and 2.0 requires an email field to validate. Use a fake email address set in smartfeed_constants.php unless in user profile it says it is okay to show email address.
				{
					$item->author = ($row['user_viewemail'] == '1') ? $row['user_email'] . ' (' . $row['username']  . ')' : $lang['smartfeed_fake_email'] . ' (' . $row['username'] . ')';
				}					
				else
				{
					$item->author = $row['username'];
				}
				
				// Code from rss.php by Sascha Carlin, slightly modified 
				$post_text = '<b>' . $row['username'] . '</b> ' . $lang['smartfeed_wrote'] . ' ' . $lang['smartfeed_at'] . ' <b>' . $row['Post Time'] . ' ' . $l_timezone . '</b>: ' . $row['post_text'];
				$post_text = str_replace("\n", "\n<br />\n", $post_text);
				$post_text = bbencode_second_pass($post_text, $row['bbcode_uid']);
				$post_text = smilies_pass($post_text);
				$post_text = preg_replace("/$smilies_path/", $smilies_url, $post_text);
				$post_text = make_clickable($post_text);
				// Variable reassignment and reformatting for user sig
				$user_sig = $row['user_sig'];
				$user_sig = bbencode_second_pass($user_sig, $post['user_sig_bbcode_uid']);
				$user_sig = smilies_pass($user_sig);
				$user_sig = preg_replace("/$smilies_path/", $smilies_url, $user_sig);
				$user_sig = make_clickable($user_sig);
				// End Code from rss.php by Sascha Carlin, slightly modified 

				if ( $user_sig != '' )
				{
					$user_sig = '<br />_________________<br />' . str_replace("\n", "\n<br />\n", $user_sig);
				}
				
				if ($feed_type=='RSS0.91') // RSS 0.91 requires that link and description may each not exceed 500 characters and are not handled by the class.
				{
					$item->link = substr($siteURL . 'viewtopic.' . $phpEx . '?' . POST_POST_URL . '=' . $row['post_id'] . '#' . $row['post_id'],0,500);
					$item->description = substr($post_text . $user_sig,0,500);
				}
				else
				{
					$item->link = $siteURL . 'viewtopic.' . $phpEx . '?' . POST_POST_URL . '=' . $row['post_id'] . '#' . $row['post_id'];
					$item->description = $post_text. $user_sig;
				}
				
				$rss->addItem($item);

			}
			$rss->outputFeed($feed_type);				
			$db->sql_freeresult ($result);
		}
		
		// If there were no problems, reset the user's last visit date to the current time and date if they so elected. By doing so
		// when they next hit the phpBB board it should appear as if all messages were read.
		
		if ($reset)
		{
			$sql = "UPDATE " . USERS_TABLE . "
				SET USER_LASTVISIT = UNIX_TIMESTAMP(NOW())
				WHERE user_id = '" . $user_id . "'";
				
			if ( !($result = $db->sql_query($sql)) )
			{
				$error_lang = $lang['smartfeed_reset_error'];
				$error = true;
			}
		}
	}
}

if ($error)
{
    // If there was an error, show it as HTML.
	//$userdata = session_pagestart($user_ip, PAGE_INDEX);
	//init_userprefs($userdata);
	include($phpbb_root_path . 'includes/page_header.'.$phpEx);
	$template->set_filenames(array('smartfeed' => 'smartfeed_error_body.tpl'));
	$template->assign_vars(array(
		'PAGE_TITLE' => $lang['smartfeed_error_title'],
		'ERROR_INTRODUCTION' => $lang['smartfeed_error_introduction'],
		'ERROR_MESSAGE' => $error_lang));
	$template->pparse('smartfeed');
	include($phpbb_root_path . 'includes/page_tail.'.$phpEx);
}

?>
